# Copyright (C) 2019  Nexedi SA and Contributors.
#                     Romain Courteaud <romain@nexedi.com>
#
# This program is free software: you can Use, Study, Modify and Redistribute
# it under the terms of the GNU General Public License version 3, or (at your
# option) any later version, as published by the Free Software Foundation.
#
# You can also Link and Combine this program with other software covered by
# the terms of any of the Free Software licenses or any of the Open Source
# Initiative approved licenses and Convey the resulting work. Corresponding
# source of such a combination shall include the source code for all other
# software used.
#
# This program is distributed WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See COPYING file for full licensing terms.
# See https://www.nexedi.com/licensing for rationale and options.

import unittest
from surykatka.db import LogDB
import surykatka.ssl
from surykatka.ssl import (
    logSslCertificate,
    hasValidSSLCertificate,
    packSslCertificate,
)
from surykatka.status import logStatus
import mock
import peewee
import datetime


class TZ(datetime.tzinfo):
    """A time zone with an arbitrary, constant -06:39 offset."""

    def utcoffset(self, dt):
        return datetime.timedelta(hours=-6, minutes=-39)

    def dst(self, dt):
        return datetime.timedelta(hours=-6, minutes=-39)


class SurykatkaSslTestCase(unittest.TestCase):
    def setUp(self):
        self.db = LogDB(":memory:")
        self.db.createTables()

    ################################################
    # logSSLCertificate
    ################################################
    def test_logSSLCertificate_insertFirst(self):
        ip = "127.0.0.1"
        port = 1234
        hostname = "example.org"
        status_id = logStatus(self.db, "foo")
        sha1_fingerprint = "asdfghj"
        not_before = datetime.datetime.utcnow()
        not_after = datetime.datetime(2000, 1, 1, tzinfo=TZ())
        subject = "foosubject"
        issuer = "barissuer"

        result = logSslCertificate(
            self.db,
            ip,
            port,
            hostname,
            sha1_fingerprint,
            not_before,
            not_after,
            subject,
            issuer,
            status_id,
        )

        assert self.db.SslChange.select().count() == 1
        assert self.db.SslChange.get().ip == ip
        assert self.db.SslChange.get().port == port
        assert self.db.SslChange.get().hostname == hostname
        assert self.db.SslChange.get().sha1_fingerprint == sha1_fingerprint
        assert (
            int(
                (
                    self.db.SslChange.get().not_before - not_before
                ).total_seconds()
            )
            == 0
        )
        assert self.db.SslChange.get().not_after == datetime.datetime(
            2000, 1, 1, 6, 39
        )

        assert self.db.SslChange.get().subject == subject
        assert self.db.SslChange.get().issuer == issuer
        assert result == status_id

    def test_logSSLCertificate_insertOnlyOnePerStatusIdIPPortHostname(self):
        ip = "127.0.0.1"
        port = 1234
        hostname = "example.org"
        status_id = logStatus(self.db, "foo")
        sha1_fingerprint = "asdfghj"
        not_before = datetime.datetime.utcnow()
        not_after = datetime.datetime.utcnow()
        subject = "foosubject"
        issuer = "barissuer"

        logSslCertificate(
            self.db,
            ip,
            port,
            hostname,
            sha1_fingerprint,
            not_before,
            not_after,
            subject,
            issuer,
            status_id,
        )

        try:
            logSslCertificate(
                self.db,
                ip,
                port,
                hostname,
                sha1_fingerprint + ".",
                not_before,
                not_after,
                subject,
                issuer,
                status_id,
            )
        except peewee.IntegrityError:
            assert self.db.SslChange.select().count() == 1
            assert self.db.SslChange.get().status_id == status_id
        else:
            raise NotImplementedError("Expected IntegrityError")

    def test_logSSLCertificate_skipIdenticalPreviousValues(self):
        ip = "127.0.0.1"
        port = 1234
        hostname = "example.org"
        sha1_fingerprint = "asdfghj"
        not_before = datetime.datetime.utcnow()
        not_after = datetime.datetime.utcnow()
        subject = "foosubject"
        issuer = "barissuer"

        status_id = logStatus(self.db, "foo")
        result = logSslCertificate(
            self.db,
            ip,
            port,
            hostname,
            sha1_fingerprint,
            not_before,
            not_after,
            subject,
            issuer,
            status_id,
        )

        status_id_2 = logStatus(self.db, "foo")
        result_2 = logSslCertificate(
            self.db,
            ip,
            port,
            hostname,
            sha1_fingerprint,
            not_before,
            not_after,
            subject,
            issuer,
            status_id_2,
        )

        assert result_2 == result
        assert self.db.SslChange.select().count() == 1
        assert self.db.SslChange.get().ip == ip
        assert self.db.SslChange.get().port == port
        assert self.db.SslChange.get().hostname == hostname
        assert self.db.SslChange.get().sha1_fingerprint == sha1_fingerprint
        assert self.db.SslChange.get().status_id == status_id

    def test_logSSLCertificate_insertWhenDifferentState(self):
        ip = "127.0.0.1"
        port = 1234
        hostname = "example.org"
        sha1_fingerprint = "asdfghj"
        not_before = datetime.datetime.utcnow()
        not_after = datetime.datetime.utcnow()
        subject = "foosubject"
        issuer = "barissuer"

        status_id = logStatus(self.db, "foo")
        result = logSslCertificate(
            self.db,
            ip,
            port,
            hostname,
            sha1_fingerprint,
            not_before,
            not_after,
            subject,
            issuer,
            status_id,
        )

        status_id_2 = logStatus(self.db, "foo")
        sha1_fingerprint_2 = sha1_fingerprint + "."
        result_2 = logSslCertificate(
            self.db,
            ip,
            port,
            hostname,
            sha1_fingerprint_2,
            not_before,
            not_after,
            subject,
            issuer,
            status_id_2,
        )

        assert result_2 != result
        assert self.db.SslChange.select().count() == 2
        assert (
            self.db.SslChange.get(
                self.db.SslChange.status == status_id
            ).sha1_fingerprint
            == sha1_fingerprint
        )
        assert (
            self.db.SslChange.get(
                self.db.SslChange.status == status_id_2
            ).sha1_fingerprint
            == sha1_fingerprint_2
        )

    def test_logSSLCertificate_insertDifferentKeys(self):
        ip = "127.0.0.1"
        ip_2 = ip + "2"
        port = 1234
        port_2 = port + 1
        hostname = "example.org"
        hostname_2 = hostname + "."
        status_id = logStatus(self.db, "foo")
        sha1_fingerprint = "asdfghj"
        not_before = datetime.datetime.utcnow()
        not_after = datetime.datetime.utcnow()
        subject = "foosubject"
        issuer = "barissuer"

        args = [
            sha1_fingerprint,
            not_before,
            not_after,
            subject,
            issuer,
            status_id,
        ]

        logSslCertificate(self.db, ip, port, hostname, *args)
        logSslCertificate(self.db, ip_2, port, hostname, *args)
        logSslCertificate(self.db, ip, port_2, hostname, *args)
        logSslCertificate(self.db, ip, port, hostname_2, *args)
        logSslCertificate(self.db, ip_2, port_2, hostname, *args)
        logSslCertificate(self.db, ip_2, port, hostname_2, *args)
        logSslCertificate(self.db, ip, port_2, hostname_2, *args)
        logSslCertificate(self.db, ip_2, port_2, hostname_2, *args)

        assert self.db.SslChange.select().count() == 8

    ################################################
    # hasValidSSLCertificate
    ################################################
    def test_hasValidSSLCertificate_valid(self):
        ip = "127.0.0.1"
        port = 1234
        hostname = "example.org"
        status_id = logStatus(self.db, "foo")

        with mock.patch(
            "surykatka.ssl.socket.socket"
        ) as mock_socket, mock.patch(
            "surykatka.ssl.ssl.create_default_context"
        ) as mock_create_context:
            mock_create_context.return_value.wrap_socket.return_value.getpeercert.side_effect = [
                b"",
                {
                    "notBefore": "Jan 27 04:33:22 2020 GMT",
                    "notAfter": "Jan 27 04:33:22 2020 GMT",
                    "subject": [[("commonName", "foo")]],
                    "issuer": [[("commonName", "bar")]],
                },
            ]
            result = hasValidSSLCertificate(
                self.db, ip, port, hostname, status_id
            )

            assert mock_socket.call_count == 1
            assert mock_socket.return_value.settimeout.call_count == 1
            mock_socket.return_value.settimeout.assert_called_with(2)
            assert mock_socket.return_value.connect.call_count == 0
            assert mock_socket.return_value.close.call_count == 0

            assert mock_create_context.call_count == 1
            assert mock_create_context.return_value.wrap_socket.call_count == 1
            mock_create_context.return_value.wrap_socket.assert_called_with(
                mock_socket.return_value, server_hostname=hostname
            )

            assert (
                mock_create_context.return_value.wrap_socket.return_value.connect.call_count
                == 1
            )
            mock_create_context.return_value.wrap_socket.return_value.connect.assert_called_with(
                (ip, port)
            )
            assert (
                mock_create_context.return_value.wrap_socket.return_value.getpeercert.call_count
                == 2
            )
            assert mock_create_context.return_value.wrap_socket.return_value.getpeercert.call_args_list[
                0
            ] == mock.call(
                True
            )
            assert (
                mock_create_context.return_value.wrap_socket.return_value.getpeercert.call_args_list[
                    1
                ]
                == mock.call()
            )

        assert self.db.SslChange.select().count() == 1
        assert self.db.SslChange.get().ip == ip
        assert self.db.SslChange.get().port == port
        assert self.db.SslChange.get().hostname == hostname
        assert (
            self.db.SslChange.get().sha1_fingerprint
            == "da39a3ee5e6b4b0d3255bfef95601890afd80709"
        )
        assert self.db.SslChange.get().not_before == datetime.datetime(
            2020, 1, 27, 4, 33, 22
        )
        assert self.db.SslChange.get().not_after == datetime.datetime(
            2020, 1, 27, 4, 33, 22
        )
        assert self.db.SslChange.get().subject == "foo"
        assert self.db.SslChange.get().issuer == "bar"
        assert self.db.SslChange.get().status_id == status_id
        assert result == True

    def test_hasValidSSLCertificate_validTwice(self):
        ip = "127.0.0.1"
        port = 1234
        hostname = "example.org"
        status_id = logStatus(self.db, "foo")
        status_id_2 = logStatus(self.db, "foo")

        with mock.patch(
            "surykatka.ssl.socket.socket"
        ) as mock_socket, mock.patch(
            "surykatka.ssl.ssl.create_default_context"
        ) as mock_create_context:
            mock_create_context.return_value.wrap_socket.return_value.getpeercert.side_effect = [
                b"",
                {
                    "notBefore": "Jan 27 04:33:22 2020 GMT",
                    "notAfter": "Jan 27 04:33:22 2020 GMT",
                    "subject": [[("commonName", "foo")]],
                    "issuer": [[("commonName", "bar")]],
                },
            ] * 2
            result = hasValidSSLCertificate(
                self.db, ip, port, hostname, status_id
            )
            result = hasValidSSLCertificate(
                self.db, ip, port, hostname, status_id_2
            )

            assert mock_socket.call_count == 2
            assert mock_socket.return_value.settimeout.call_count == 2
            mock_socket.return_value.settimeout.assert_called_with(2)
            assert mock_socket.return_value.connect.call_count == 0
            assert mock_socket.return_value.close.call_count == 0

            assert mock_create_context.call_count == 2
            assert mock_create_context.return_value.wrap_socket.call_count == 2
            mock_create_context.return_value.wrap_socket.assert_called_with(
                mock_socket.return_value, server_hostname=hostname
            )

            assert (
                mock_create_context.return_value.wrap_socket.return_value.connect.call_count
                == 2
            )
            mock_create_context.return_value.wrap_socket.return_value.connect.assert_called_with(
                (ip, port)
            )
            assert (
                mock_create_context.return_value.wrap_socket.return_value.getpeercert.call_count
                == 4
            )
            assert mock_create_context.return_value.wrap_socket.return_value.getpeercert.call_args_list[
                0
            ] == mock.call(
                True
            )
            assert (
                mock_create_context.return_value.wrap_socket.return_value.getpeercert.call_args_list[
                    1
                ]
                == mock.call()
            )

        assert self.db.SslChange.select().count() == 1
        assert self.db.SslChange.get().ip == ip
        assert self.db.SslChange.get().port == port
        assert self.db.SslChange.get().hostname == hostname
        assert (
            self.db.SslChange.get().sha1_fingerprint
            == "da39a3ee5e6b4b0d3255bfef95601890afd80709"
        )
        assert self.db.SslChange.get().not_before == datetime.datetime(
            2020, 1, 27, 4, 33, 22
        )
        assert self.db.SslChange.get().not_after == datetime.datetime(
            2020, 1, 27, 4, 33, 22
        )
        assert self.db.SslChange.get().subject == "foo"
        assert self.db.SslChange.get().issuer == "bar"
        assert self.db.SslChange.get().status_id == status_id
        assert result == True

    def test_hasValidSSLCertificate_sslError(self):
        ip = "127.0.0.1"
        port = 1234
        hostname = "example.org"
        status_id = logStatus(self.db, "foo")

        with mock.patch(
            "surykatka.ssl.socket.socket"
        ) as mock_socket, mock.patch(
            "surykatka.ssl.ssl.create_default_context"
        ) as mock_create_context:

            def sideEffect(*args, **kw):
                raise surykatka.ssl.ssl.SSLError()

            mock_create_context.return_value.wrap_socket.return_value.connect.side_effect = (
                sideEffect
            )

            result = hasValidSSLCertificate(
                self.db, ip, port, hostname, status_id
            )

            assert mock_socket.call_count == 1
            assert mock_socket.return_value.settimeout.call_count == 1
            mock_socket.return_value.settimeout.assert_called_with(2)
            assert mock_socket.return_value.connect.call_count == 0
            assert mock_socket.return_value.close.call_count == 0

            assert mock_create_context.call_count == 1
            assert mock_create_context.return_value.wrap_socket.call_count == 1
            mock_create_context.return_value.wrap_socket.assert_called_with(
                mock_socket.return_value, server_hostname=hostname
            )

            assert (
                mock_create_context.return_value.wrap_socket.return_value.connect.call_count
                == 1
            )
            mock_create_context.return_value.wrap_socket.return_value.connect.assert_called_with(
                (ip, port)
            )
            assert (
                mock_create_context.return_value.wrap_socket.return_value.getpeercert.call_count
                == 0
            )

        assert self.db.SslChange.select().count() == 1
        assert self.db.SslChange.get().ip == ip
        assert self.db.SslChange.get().port == port
        assert self.db.SslChange.get().hostname == hostname
        assert self.db.SslChange.get().sha1_fingerprint == None
        assert self.db.SslChange.get().not_before == None
        assert self.db.SslChange.get().not_after == None
        assert self.db.SslChange.get().subject == None
        assert self.db.SslChange.get().issuer == None
        assert self.db.SslChange.get().status_id == status_id
        assert result == False

    def test_hasValidSSLCertificate_ConnectionRefusedError(self):
        ip = "127.0.0.1"
        port = 1234
        hostname = "example.org"
        status_id = logStatus(self.db, "foo")

        with mock.patch(
            "surykatka.ssl.socket.socket"
        ) as mock_socket, mock.patch(
            "surykatka.ssl.ssl.create_default_context"
        ) as mock_create_context:

            def sideEffect(*args, **kw):
                raise ConnectionRefusedError()

            mock_create_context.return_value.wrap_socket.return_value.connect.side_effect = (
                sideEffect
            )

            result = hasValidSSLCertificate(
                self.db, ip, port, hostname, status_id
            )

            assert mock_socket.call_count == 1
            assert mock_socket.return_value.settimeout.call_count == 1
            mock_socket.return_value.settimeout.assert_called_with(2)
            assert mock_socket.return_value.connect.call_count == 0
            assert mock_socket.return_value.close.call_count == 0

            assert mock_create_context.call_count == 1
            assert mock_create_context.return_value.wrap_socket.call_count == 1
            mock_create_context.return_value.wrap_socket.assert_called_with(
                mock_socket.return_value, server_hostname=hostname
            )

            assert (
                mock_create_context.return_value.wrap_socket.return_value.connect.call_count
                == 1
            )
            mock_create_context.return_value.wrap_socket.return_value.connect.assert_called_with(
                (ip, port)
            )
            assert (
                mock_create_context.return_value.wrap_socket.return_value.getpeercert.call_count
                == 0
            )

        assert self.db.SslChange.select().count() == 1
        assert self.db.SslChange.get().ip == ip
        assert self.db.SslChange.get().port == port
        assert self.db.SslChange.get().hostname == hostname
        assert self.db.SslChange.get().sha1_fingerprint == None
        assert self.db.SslChange.get().not_before == None
        assert self.db.SslChange.get().not_after == None
        assert self.db.SslChange.get().subject == None
        assert self.db.SslChange.get().issuer == None
        assert self.db.SslChange.get().status_id == status_id
        assert result == False

    def test_hasValidSSLCertificate_timeout(self):
        ip = "127.0.0.1"
        port = 1234
        hostname = "example.org"
        status_id = logStatus(self.db, "foo")

        with mock.patch(
            "surykatka.ssl.socket.socket"
        ) as mock_socket, mock.patch(
            "surykatka.ssl.ssl.create_default_context"
        ) as mock_create_context:

            def sideEffect(*args, **kw):
                raise surykatka.ssl.socket.timeout()

            mock_create_context.return_value.wrap_socket.return_value.connect.side_effect = (
                sideEffect
            )

            result = hasValidSSLCertificate(
                self.db, ip, port, hostname, status_id
            )

            assert mock_socket.call_count == 1
            assert mock_socket.return_value.settimeout.call_count == 1
            mock_socket.return_value.settimeout.assert_called_with(2)
            assert mock_socket.return_value.connect.call_count == 0
            assert mock_socket.return_value.close.call_count == 0

            assert mock_create_context.call_count == 1
            assert mock_create_context.return_value.wrap_socket.call_count == 1
            mock_create_context.return_value.wrap_socket.assert_called_with(
                mock_socket.return_value, server_hostname=hostname
            )

            assert (
                mock_create_context.return_value.wrap_socket.return_value.connect.call_count
                == 1
            )
            mock_create_context.return_value.wrap_socket.return_value.connect.assert_called_with(
                (ip, port)
            )
            assert (
                mock_create_context.return_value.wrap_socket.return_value.getpeercert.call_count
                == 0
            )

        assert self.db.SslChange.select().count() == 1
        assert self.db.SslChange.get().ip == ip
        assert self.db.SslChange.get().port == port
        assert self.db.SslChange.get().hostname == hostname
        assert self.db.SslChange.get().sha1_fingerprint == None
        assert self.db.SslChange.get().not_before == None
        assert self.db.SslChange.get().not_after == None
        assert self.db.SslChange.get().subject == None
        assert self.db.SslChange.get().issuer == None
        assert self.db.SslChange.get().status_id == status_id
        assert result == False

    def test_hasValidSSLCertificate_OSError(self):
        ip = "127.0.0.1"
        port = 1234
        hostname = "example.org"
        status_id = logStatus(self.db, "foo")

        with mock.patch(
            "surykatka.ssl.socket.socket"
        ) as mock_socket, mock.patch(
            "surykatka.ssl.ssl.create_default_context"
        ) as mock_create_context:

            def sideEffect(*args, **kw):
                raise OSError()

            mock_create_context.return_value.wrap_socket.return_value.connect.side_effect = (
                sideEffect
            )

            result = hasValidSSLCertificate(
                self.db, ip, port, hostname, status_id
            )

            assert mock_socket.call_count == 1
            assert mock_socket.return_value.settimeout.call_count == 1
            mock_socket.return_value.settimeout.assert_called_with(2)
            assert mock_socket.return_value.connect.call_count == 0
            assert mock_socket.return_value.close.call_count == 0

            assert mock_create_context.call_count == 1
            assert mock_create_context.return_value.wrap_socket.call_count == 1
            mock_create_context.return_value.wrap_socket.assert_called_with(
                mock_socket.return_value, server_hostname=hostname
            )

            assert (
                mock_create_context.return_value.wrap_socket.return_value.connect.call_count
                == 1
            )
            mock_create_context.return_value.wrap_socket.return_value.connect.assert_called_with(
                (ip, port)
            )
            assert (
                mock_create_context.return_value.wrap_socket.return_value.getpeercert.call_count
                == 0
            )

        assert self.db.SslChange.select().count() == 1
        assert self.db.SslChange.get().ip == ip
        assert self.db.SslChange.get().port == port
        assert self.db.SslChange.get().hostname == hostname
        assert self.db.SslChange.get().sha1_fingerprint == None
        assert self.db.SslChange.get().not_before == None
        assert self.db.SslChange.get().not_after == None
        assert self.db.SslChange.get().subject == None
        assert self.db.SslChange.get().issuer == None
        assert self.db.SslChange.get().status_id == status_id
        assert result == False

    def test_hasValidSSLCertificate_Exception(self):
        ip = "127.0.0.1"
        port = 1234
        hostname = "example.org"
        status_id = logStatus(self.db, "foo")

        with mock.patch(
            "surykatka.ssl.socket.socket"
        ) as mock_socket, mock.patch(
            "surykatka.ssl.ssl.create_default_context"
        ) as mock_create_context:

            def sideEffect(*args, **kw):
                raise Exception()

            mock_create_context.return_value.wrap_socket.return_value.connect.side_effect = (
                sideEffect
            )

            try:
                hasValidSSLCertificate(self.db, ip, port, hostname, status_id)
            except Exception:
                assert self.db.SslChange.select().count() == 0
            else:
                raise NotImplementedError("Expected OSError")

            assert mock_socket.call_count == 1
            assert mock_socket.return_value.settimeout.call_count == 1
            mock_socket.return_value.settimeout.assert_called_with(2)
            assert mock_socket.return_value.connect.call_count == 0
            assert mock_socket.return_value.close.call_count == 0

            assert mock_create_context.call_count == 1
            assert mock_create_context.return_value.wrap_socket.call_count == 1
            mock_create_context.return_value.wrap_socket.assert_called_with(
                mock_socket.return_value, server_hostname=hostname
            )

            assert (
                mock_create_context.return_value.wrap_socket.return_value.connect.call_count
                == 1
            )
            mock_create_context.return_value.wrap_socket.return_value.connect.assert_called_with(
                (ip, port)
            )
            assert (
                mock_create_context.return_value.wrap_socket.return_value.getpeercert.call_count
                == 0
            )

    ################################################
    # packSslCertificate
    ################################################
    def test_packSslCertificate_differentState(self):
        ip = "127.0.0.1"
        port = 1234
        hostname = "example.org"
        sha1_fingerprint = "asdfghj"
        not_before = datetime.datetime.utcnow()
        not_after = datetime.datetime.utcnow()
        subject = "foosubject"
        issuer = "barissuer"

        status_id = logStatus(self.db, "foo")
        logSslCertificate(
            self.db,
            ip,
            port,
            hostname,
            sha1_fingerprint,
            not_before,
            not_after,
            subject,
            issuer,
            status_id,
        )

        status_id_2 = logStatus(self.db, "foo")
        sha1_fingerprint_2 = sha1_fingerprint + "."
        logSslCertificate(
            self.db,
            ip,
            port,
            hostname,
            sha1_fingerprint_2,
            not_before,
            not_after,
            subject,
            issuer,
            status_id_2,
        )

        result = packSslCertificate(self.db)
        assert self.db.SslChange.select().count() == 1
        assert self.db.SslChange.get().ip == ip
        assert self.db.SslChange.get().port == port
        assert self.db.SslChange.get().hostname == hostname
        assert self.db.SslChange.get().sha1_fingerprint == sha1_fingerprint_2
        assert self.db.SslChange.get().status_id == status_id_2
        assert result == None

    def test_packSslCertificate_keepDifferentKeys(self):
        ip = "127.0.0.1"
        ip_2 = ip + "2"
        port = 1234
        port_2 = port + 1
        hostname = "example.org"
        hostname_2 = hostname + "."
        status_id = logStatus(self.db, "foo")
        sha1_fingerprint = "asdfghj"
        not_before = datetime.datetime.utcnow()
        not_after = datetime.datetime.utcnow()
        subject = "foosubject"
        issuer = "barissuer"

        args = [
            sha1_fingerprint,
            not_before,
            not_after,
            subject,
            issuer,
            status_id,
        ]

        logSslCertificate(self.db, ip, port, hostname, *args)
        logSslCertificate(self.db, ip_2, port, hostname, *args)
        logSslCertificate(self.db, ip, port_2, hostname, *args)
        logSslCertificate(self.db, ip, port, hostname_2, *args)
        logSslCertificate(self.db, ip_2, port_2, hostname, *args)
        logSslCertificate(self.db, ip_2, port, hostname_2, *args)
        logSslCertificate(self.db, ip, port_2, hostname_2, *args)
        logSslCertificate(self.db, ip_2, port_2, hostname_2, *args)

        assert self.db.SslChange.select().count() == 8


def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(SurykatkaSslTestCase))
    return suite


if __name__ == "__main__":
    unittest.main(defaultTest="suite")
